import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { ProductReviewRepository } from "./product-review.repository";
import { CreateProductReviewDto } from "./dto/create-product-review.dto";
import { ProductReview } from "./entities/product-review.entity";
import { UpdateProductReviewDto } from "./dto/update-product-review.dto";
import { ProductsRepository } from "../product/product.repository";
import { Product } from "../product/entity/product.entity";

@Injectable()
export class ProductReviewService {
  constructor(
    @InjectRepository(ProductReviewRepository)
    private readonly repository: ProductReviewRepository,
    @InjectRepository(ProductsRepository)
    private readonly productsRepository: ProductsRepository,
  ) {
  }

  async create(
    dto: CreateProductReviewDto
  ): Promise<ProductReview | Error> {
    const data = await this.repository.createEntity(dto);
    if (!data) {
      throw new HttpException(
        Constants.CREATE_FAILED,
        HttpStatus.BAD_REQUEST
      );
    }
    const ratingData = await this.ratingCalculate((data as ProductReview)?.productId)
    const formattedResponse = {
      ...data,
      ...ratingData
    }
    return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", formattedResponse);
  }

  async ratingCalculate(productId: string): Promise<Object> {
    const data = await this.repository.findByFilterQuery({ productId: productId });
    const reviewCount = Object.keys(data).length != 0 ? Object.keys(data).length : 0;
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    let ratingAvarage = 0;
    if (Object.keys(data).length > 0) {
      let count = 0;
      for (let i = 0; i < reviewCount; i++) {
        count += parseInt(data[i].rating);
      }
      ratingAvarage = (count / reviewCount);
    }

    const updateProductDto = {
      averageRating: ratingAvarage.toFixed(2),
      reviewCount: reviewCount,
    }

    const updatedProduct = await this.productsRepository.updateEntity(productId, updateProductDto);

    return {
      reviewCount: (updatedProduct as Product).reviewCount,
      averageRating: (updatedProduct as Product).averageRating
    };
  }

  async findAll(): Promise<ProductReview[] | Error> {
    const data = await this.repository.findByFilterQuery({});
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async findOne(id: string): Promise<ProductReview[] | Error> {
    const data = await this.repository.findByFilterQuery({ productId: id });
    const reviewCount = Object.keys(data).length != 0 ? Object.keys(data).length : 0;
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    let ratingAvarage = 0;
    if (Object.keys(data).length > 0) {
      let count = 0;
      for (let i = 0; i < reviewCount; i++) {
        count += parseInt(data[i].rating);
      }
      ratingAvarage = (count / reviewCount);
    }

    const response = {
      data: data,
      reviewCount: reviewCount,
      ratingAvarage: ratingAvarage.toFixed(2)
    };
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
  }

  async update(id: string, dto: UpdateProductReviewDto): Promise<ProductReview | Error> {
    const productReview = await this.repository.findOneEntity(id);
    if (!productReview) {
      throw new HttpException(
        "Product color not found.",
        HttpStatus.NOT_FOUND
      );
    }
    const data = await this.repository.updateEntity(id, dto);
    if (!data) {
      throw new HttpException(
        Constants.UPDATE_FAILED,
        HttpStatus.BAD_REQUEST
      );
    }
    const ratingData = await this.ratingCalculate((data as ProductReview)?.productId)
    const formattedResponse = {
      ...data,
      ...ratingData
    }
    return ResponseUtils.successResponseHandler(200, "Product color updated successfully", "data", formattedResponse);
  }

  async remove(id: string): Promise<boolean | Error> {
    const data = await this.repository.deleteEntity(id);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
  }
}
