import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Query,
    UploadedFile,
    UploadedFiles,
    UseInterceptors
} from '@nestjs/common';
import { FileFieldsInterceptor, FileInterceptor } from '@nestjs/platform-express';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { Constants } from 'src/utils/constants';
import { CreateProductDto } from './dto/create-product.dto';
import { UpdateProductDto } from './dto/update-product.dto';
import { ProductService } from './product.service';
import { getMulterConfig } from 'src/core/multer.config';
import { CreateImportProductDto } from './dto/create-import-product.dto';
import { CreateConfirmImportProductDto } from './dto/create-confirm-import-product.dto';
import { CreateConfirmStockInProductDto } from './dto/create-confirm-stockin-product.dto';

@Controller({
    path: 'product',
    version: Constants.API_VERSION_1
})
export class ProductController {

    constructor(
        private readonly productsService: ProductService,
    ) {
    }


    @Post()
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'productImages', maxCount: 4 },
            { name: 'featuredImage', maxCount: 1 },
        ],
    ))
    create(
        @Body() dto: CreateProductDto,
        @UploadedFiles() files: {
            productImages?: UploadedMulterFileI[],
            featuredImage?: UploadedMulterFileI,
        }) {
           
           
           
        return this.productsService.create(dto, files);
    }

    // @Get('/')
    // getAll(
    //     @Query('page') pageParam: string,
    //     @Query('limit') limitParam: string
    // ) {
    //     const page = parseInt(pageParam, 10) || 1;
    //     const limit = parseInt(limitParam, 10) || 10;
    //     return this.productsService.findAll(page, limit);
    // }

    @Get()
    async findAll(
        @Query('limit') limit: number,
        @Query('page') page: number
    ) {
        return await this.productsService.findAll(limit, page);
    }

    @Get('admin')
    getAllForAdmin() {
        return this.productsService.getAllForAdmin();
    }

    @Get('/:id') // for public viewer
    async findOne(@Param('id') id: string) {
        return await this.productsService.findOne(id);
    }

    @Get('/product-details/:slug')
    async findOneBySlug(@Param('slug') slug: string) {
        return await this.productsService.findOneBySlug(slug);
    }

    @Patch('/:id')
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'productImages', maxCount: 4 },
            { name: 'featuredImage', maxCount: 1 },
        ],
    ))
    async updateProduct(
        @Param('id') id: string,
        @Body() updateProductDto: UpdateProductDto,
        @UploadedFiles() files: {
            productImages?: UploadedMulterFileI[],
            featuredImage?: UploadedMulterFileI
        }
    ) {
        return await this.productsService.updateOneProduct(id, updateProductDto, files);
    }

    @Delete(':id')
    async remove(@Param('id') id: string) {
        return await this.productsService.remove(id);
    }

    @Post('/import-product')
    @UseInterceptors(
        FileInterceptor(
            'excelFile',
            getMulterConfig(
                './uploads/product-dump'
            )
        )
    )
    importProduct(@Body() importProductDto: CreateImportProductDto, @UploadedFile() file: Express.Multer.File) {
        return this.productsService.importProductService(importProductDto, file);
    }

    @Post('/confirm-import-product')
    confirmImportProduct(@Body() importProductDto: CreateConfirmImportProductDto) {
        
        return this.productsService.confirmImportProductService(importProductDto);
    }

    @Post('/new-stock-in')
    @UseInterceptors(
        FileInterceptor(
            'excelFile',
            getMulterConfig(
                './uploads/product-stockin'
            )
        )
    )
    importStockInProduct(@Body() importProductDto: CreateImportProductDto, @UploadedFile() file: Express.Multer.File) {
        return this.productsService.importProductStockService(importProductDto, file);
    }

    @Post('/confirm-stockin-product')
    confirmStockinProduct(@Body() dto: CreateConfirmStockInProductDto) {
        return this.productsService.confirmStockInProductService(dto);
    }
}
