import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { ConsoleLogger } from '@nestjs/common/services';
import { InjectRepository } from '@nestjs/typeorm';
import { DirectPost } from 'src/helper/nmi-payment-gatway-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { OrderRepository } from '../orders/order/order-repository';
import { PaymentConfigurationRepository } from '../payment-configuration/payment-configuration.repository';
import { PaymentGatewayRepository } from '../payment-gateway/payment-gateway.repository';
import { ProductsRepository } from '../product/product/product.repository';
import { RefundItemRepository } from '../refund-item/refund-item.repository';
import { CreateReturnProductAdminDto } from './dto/create-return-product-admin.dto';
import { CreateReturnProductDto } from './dto/create-return-product.dto';
import { UpdateReturnProductDto } from './dto/update-return-product.dto';
import { ReturnProduct } from './entities/return-product.entity';
import { ReturnProductsRepository } from './return-products.repository';

@Injectable()
export class ReturnProductsService {
  constructor(
    @InjectRepository(ReturnProductsRepository)
    private readonly repository: ReturnProductsRepository,
    @InjectRepository(PaymentConfigurationRepository)
    private readonly paymentConfigurationRepository: PaymentConfigurationRepository,
    @InjectRepository(PaymentGatewayRepository)
    private readonly paymentGatewayRepository: PaymentGatewayRepository,
    @InjectRepository(RefundItemRepository)
    private readonly refundItemRepository: RefundItemRepository,
    @InjectRepository(ProductsRepository)
    private readonly productRepo: ProductsRepository,
    @InjectRepository(OrderRepository)
    private readonly orderRepo: OrderRepository,
  ) {}

  async create(dto: CreateReturnProductDto) {
    await dto.products.forEach(async (item: any) => {
      const productDto = {
        orderId: dto.orderId,
        customerId: dto.customerId,
        customerName: dto.customerName,
        productId: item.id,
        productName: item.name,
        quantity: item.quantity,
        actualPrice: item.actualPrice,
        totalPrice: parseFloat(item.actualPrice) * parseInt(item.quantity),
        note: item.note,
      };

      const data = await this.repository.createEntity(productDto);
    });

    return ResponseUtils.successResponseHandler(
      201,
      'Data created successfully',
      'data',
      [],
    );
  }

  async adminReturn(dto: CreateReturnProductAdminDto) {
    const data = await this.repository.createEntity(dto);
    if (!data) {
      throw new HttpException(Constants.CREATE_FAILED, HttpStatus.BAD_REQUEST);
    }

    // find order
    const order = await this.orderRepo.findOne({
      where: {
        orderId: dto.orderId,
        isDeleted: false,
      },
    });

    if (order.paymentMethodName === 'Cash on delivery') {
      // fetch payement info using invoiceid
      const paymentGateway = await this.paymentGatewayRepository.findOne({
        where: {
          invoiceID: dto.orderId,
          isDeleted: false,
        },
      });

      if (!paymentGateway) {
        throw new HttpException(
          'Payment Gateway Not Found!',
          HttpStatus.NOT_FOUND,
        );
      }

      const payload = {
        isRefunded: true,
        refundDate: new Date(),
        refundAmount:
          Number(dto.totalPrice) + Number(paymentGateway.refundAmount),
      };

      const res = await this.paymentGatewayRepository.updateEntity(
        paymentGateway.id,
        payload,
      );

      const refundItem = await this.refundItemRepository.createEntity({
        invoiceID: dto.orderId,
        refundAmount: dto.totalPrice,
        isRefunded: true,
        refundDate: new Date(),
        refundProductID: dto.productId,
      });

      // fetch single product
      const product = await this.productRepo.findOne({
        where: {
          id: dto.productId,
          isDeleted: false,
        },
      });

      if (product) {
        // update product stock
        const increaseStock = product['stock'] + dto.quantity;
        const updateProductInventory = await this.productRepo.update(
          product.id,
          {
            stock: increaseStock,
          },
        );
      }
    } else {
      // fetch payment configuration
      const paymentConf = await this.paymentConfigurationRepository.find();

      if (!paymentConf) {
        throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
      }

      // fetch payement info using invoiceid
      const paymentGateway = await this.paymentGatewayRepository.findOne({
        where: {
          invoiceID: dto.orderId,
          isDeleted: false,
        },
      });

      if (!paymentGateway) {
        throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
      }

      // call class constarctor
      const dp = new DirectPost(paymentConf[0].securityKey);

      // refund amount
      const refund = await dp.doRefund(
        dto.totalPrice,
        paymentGateway.paymentTransID,
      );
      if (refund['statusCode'] === 200) {
        const payload = {
          refundTransID: refund['transactionid'],
          refundPayload: JSON.stringify(refund),
          isRefunded: true,
          refundDate: new Date(),
          refundAmount:
            Number(dto.totalPrice) + Number(paymentGateway.refundAmount),
        };

        const res = await this.paymentGatewayRepository.updateEntity(
          paymentGateway.id,
          payload,
        );

        if (res) {
          const refundItem = await this.refundItemRepository.createEntity({
            invoiceID: dto.orderId,
            refundAmount: dto.totalPrice,
            isRefunded: true,
            refundDate: new Date(),
            refundProductID: dto.productId,
          });
        }
      } else {
        (refund);
      }

      // fetch single product
      const product = await this.productRepo.findOne({
        where: {
          id: dto.productId,
          isDeleted: false,
        },
      });

      if (product) {
        // update product stock
        const increaseStock = product['stock'] + dto.quantity;
        const updateProductInventory = await this.productRepo.update(
          dto.productId,
          {
            stock: increaseStock,
          },
        );
      }
    }

    return ResponseUtils.successResponseHandler(
      201,
      'Data created successfully',
      'data',
      [],
    );
  }

  async update(id: string, dto: any): Promise<ReturnProduct | Error> {
    // find return products
    const returnProduts = await this.repository.findOneEntity(id);

    if (!returnProduts) {
      throw new HttpException(
        'Return Products Not Found!',
        HttpStatus.NOT_FOUND,
      );
    }

    // update return products status
    const data = await this.repository.updateEntity(id, dto);

    if (!data) {
      throw new HttpException('Update not found!', HttpStatus.NOT_FOUND);
    }

    if (data['status'] === 'Approved') {
      // find order
      const order = await this.orderRepo.findOne({
        where: {
          orderId: returnProduts['orderId'],
          isDeleted: false,
        },
      });

      if (order.paymentMethodName === 'Cash on delivery') {
        // fetch payement info using invoiceid
        const paymentGateway = await this.paymentGatewayRepository.findOne({
          where: {
            invoiceID: returnProduts['orderId'],
            isDeleted: false,
          },
        });

        if (!paymentGateway) {
          throw new HttpException(
            'Payment Gateway Not Found!',
            HttpStatus.NOT_FOUND,
          );
        }

        const payload = {
          isRefunded: true,
          refundDate: new Date(),
          refundAmount:
            Number(returnProduts['totalPrice']) +
            Number(paymentGateway.refundAmount),
        };

        const res = await this.paymentGatewayRepository.updateEntity(
          paymentGateway.id,
          payload,
        );

        const refundItem = await this.refundItemRepository.createEntity({
          invoiceID: returnProduts['orderId'],
          refundAmount: returnProduts['totalPrice'],
          isRefunded: true,
          refundDate: new Date(),
          refundProductID: id,
        });

        // fetch single product
        const product = await this.productRepo.findOne({
          where: {
            id: returnProduts['productId'],
            isDeleted: false,
          },
        });

        if (product) {
          // update product stock
          const increaseStock = product['stock'] + returnProduts['quantity'];
          const updateProductInventory = await this.productRepo.update(
            product.id,
            {
              stock: increaseStock,
            },
          );
        }
      } else {
        // fetch payment configuration
        const paymentConf = await this.paymentConfigurationRepository.find();

        if (!paymentConf) {
          throw new HttpException(
            'Payment Config Not Found!',
            HttpStatus.NOT_FOUND,
          );
        }

        // fetch payement info using invoiceid
        const paymentGateway = await this.paymentGatewayRepository.findOne({
          where: {
            invoiceID: returnProduts['orderId'],
            isDeleted: false,
          },
        });

        if (!paymentGateway) {
          throw new HttpException(
            'Payment Gateway Not Found!',
            HttpStatus.NOT_FOUND,
          );
        }

        // call class constarctor
        const dp = new DirectPost(paymentConf[0].securityKey);

        // refund amount
        const refund = await dp.doRefund(
          returnProduts['totalPrice'],
          paymentGateway.paymentTransID,
        );
        if (refund['statusCode'] === 200) {
          const payload = {
            refundTransID: refund['transactionid'],
            refundPayload: JSON.stringify(refund),
            isRefunded: true,
            refundDate: new Date(),
            refundAmount:
              Number(returnProduts['totalPrice']) +
              Number(paymentGateway.refundAmount),
          };

          const res = await this.paymentGatewayRepository.updateEntity(
            paymentGateway.id,
            payload,
          );
          (res)

          if (res) {
            const refundItem = await this.refundItemRepository.createEntity({
              invoiceID: returnProduts['orderId'],
              refundAmount: returnProduts['totalPrice'],
              isRefunded: true,
              refundDate: new Date(),
              refundProductID: id,
            });
          }
        } else {
          (refund);
        }

        // fetch single product
        const product = await this.productRepo.findOne({
          where: {
            id: returnProduts['productId'],
            isDeleted: false,
          },
        });

        if (product) {
          // update product stock
          const increaseStock = product['stock'] + returnProduts['quantity'];
          const updateProductInventory = await this.productRepo.update(
            product.id,
            {
              stock: increaseStock,
            },
          );
        }
      }
    }

    return ResponseUtils.successResponseHandler(
      200,
      'Data updated successfully',
      'data',
      data,
    );
  }

  async findAll(status: string): Promise<ReturnProduct[] | Error> {
    const data = await this.repository.findByFilterQuery({ status: status });
    if (!data) {
      throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
    }
    return ResponseUtils.successResponseHandler(
      200,
      'Data fetched successfully',
      'data',
      data,
    );
  }

  async findOne(id: string): Promise<ReturnProduct[] | Error> {
    const data = await this.repository.findByFilterQuery({ id: id });
    if (!data) {
      throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
    }
    return ResponseUtils.successResponseHandler(
      200,
      'Data fetched successfully',
      'data',
      data,
    );
  }

  async remove(id: string): Promise<boolean | Error> {
    const data = await this.repository.deleteEntity(id);
    if (!data) {
      throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
    }
    return ResponseUtils.buildDeletedData(
      data,
      200,
      'Data Deleted Successfully',
    );
  }

  async bulkStatusUpdate(dto: any): Promise<ReturnProduct | Error> {
    try {
      for (let i = 0; i < dto.bulkUpdate.length; i++) {
        const returnProduct = await this.repository.findOneEntity(
          dto.bulkUpdate[i].id,
        );
        if (returnProduct) {
          const data = await this.repository.updateEntity(
            dto.bulkUpdate[i].id,
            {
              status: dto.bulkUpdate[i].status,
            },
          );

          if (data['status'] === 'Approved') {
            // find order
            const order = await this.orderRepo.findOne({
              where: {
                orderId: returnProduct['orderId'],
                isDeleted: false,
              },
            });

            if (order.paymentMethodName === 'Cash on delivery') {
              // fetch payement info using invoiceid
              const paymentGateway =
                await this.paymentGatewayRepository.findOne({
                  where: {
                    invoiceID: returnProduct['orderId'],
                    isDeleted: false,
                  },
                });

              if (!paymentGateway) {
                throw new HttpException(
                  'Payment Gateway Not Found!',
                  HttpStatus.NOT_FOUND,
                );
              }

              const payload = {
                isRefunded: true,
                refundDate: new Date(),
                refundAmount:
                  Number(returnProduct['totalPrice']) +
                  Number(paymentGateway.refundAmount),
              };

              const res = await this.paymentGatewayRepository.updateEntity(
                paymentGateway.id,
                payload,
              );

              const refundItem = await this.refundItemRepository.createEntity({
                invoiceID: returnProduct['orderId'],
                refundAmount: returnProduct['totalPrice'],
                isRefunded: true,
                refundDate: new Date(),
                refundProductID: returnProduct['ProductId'],
              });

              // fetch single product
              const product = await this.productRepo.findOne({
                where: {
                  id: returnProduct['productId'],
                  isDeleted: false,
                },
              });

              if (product) {
                // update product stock
                const increaseStock =
                  product['stock'] + returnProduct['quantity'];
                const updateProductInventory = await this.productRepo.update(
                  product.id,
                  {
                    stock: increaseStock,
                  },
                );
              }
            } else {
              // fetch payment configuration
              const paymentConf =
                await this.paymentConfigurationRepository.find();

              if (!paymentConf) {
                throw new HttpException(
                  Constants.NOT_FOUND,
                  HttpStatus.NOT_FOUND,
                );
              }

              // fetch payement info using invoiceid
              const paymentGateway =
                await this.paymentGatewayRepository.findOne({
                  where: {
                    invoiceID: returnProduct['orderId'],
                    isDeleted: false,
                  },
                });

              if (!paymentGateway) {
                throw new HttpException(
                  Constants.NOT_FOUND,
                  HttpStatus.NOT_FOUND,
                );
              }

              // call class constarctor
              const dp = new DirectPost(paymentConf[0].securityKey);

              // refund amount
              const refund = await dp.doRefund(
                returnProduct['totalPrice'],
                paymentGateway.paymentTransID,
              );
              if (refund['statusCode'] === 200) {
                // (Number(returnProduct['totalPrice']), Number(paymentGateway.refundAmount), Number(returnProduct['totalPrice']) + Number(paymentGateway.refundAmount))
                const payload = {
                  refundTransID: refund['transactionid'],
                  refundPayload: JSON.stringify(refund),
                  isRefunded: true,
                  refundDate: new Date(),
                  refundAmount:
                    Number(returnProduct['totalPrice']) +
                    Number(paymentGateway.refundAmount),
                };

                const res = await this.paymentGatewayRepository.updateEntity(
                  paymentGateway.id,
                  payload,
                );

                // ("payload = ", res)

                if (res) {
                  const refundItem =
                    await this.refundItemRepository.createEntity({
                      invoiceID: returnProduct['orderId'],
                      refundAmount: returnProduct['totalPrice'],
                      isRefunded: true,
                      refundDate: new Date(),
                      refundProductID: returnProduct['productId'],
                    });
                }
              } else {
                (refund);
              }

              // fetch single product
              const product = await this.productRepo.findOne({
                where: {
                  id: returnProduct['productId'],
                  isDeleted: false,
                },
              });

              if (product) {
                // update product stock
                const increaseStock =
                  product['stock'] + returnProduct['quantity'];
                const updateProductInventory = await this.productRepo.update(
                  product.id,
                  {
                    stock: increaseStock,
                  },
                );
              }
            }
          }
        }
      }

      return ResponseUtils.successResponseHandler(
        200,
        'Data updated successfully',
        'data',
        [],
      );
    } catch (e) {
      (e);
    }
  }

  async filterByOrderId(id: string): Promise<ReturnProduct[] | Error> {
    const data = await this.repository.findByFilterQuery({ orderId: id });
    if (!data) {
      throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
    }
    return ResponseUtils.successResponseHandler(
      200,
      'Data fetched successfully',
      'data',
      data,
    );
  }
}
